uses Windows;

const
  MaxGetEnvLen = 1024; // include last null character
  MaxExpandVarsLen = 1024; // include last null character

var
  LeadBytes: set of Char = [];

procedure InitLocale;
var
  cpinfo: TCPInfo;
  i, j: Integer;
begin
  // Initialize LeadBytes
  GetCPInfo(CP_ACP, cpinfo);
  i := 0;
  with cpinfo do
    while ((LeadByte[i] or LeadByte[i+1]) <> 0) and (i<MAX_LEADBYTES) do
    begin
      for j := LeadByte[i] to LeadByte[i+1] do
        Include(LeadBytes, Char(j));
      Inc(i, 2);
    end;
end;

function SysErrNo: Integer;
begin
  Result := GetLastError;
end;

function NumFormat(Value: Integer): string;
var
  nf: TNumberFmt;
  buf: array[0..255] of Char;
begin
  nf.NumDigits := 0;
  nf.LeadingZero := 0;
  nf.Grouping := 3;
  nf.lpDecimalSep := '.';
  nf.lpThousandSep := ',';
  nf.NegativeOrder := 1;
  GetNumberFormat(0, 0, PChar(IStr(Value)), @nf, buf, SizeOf(buf));
  Result := buf;
end;

function Lower(const S: string): string;
begin
  SetString(Result, PChar(S), Length(S));
  CharLowerBuff(PChar(Result), Length(Result));
end;

function Upper(const S: string): string;
begin
  SetString(Result, PChar(S), Length(S));
  CharUpperBuff(PChar(Result), Length(Result));
end;

function LStr(const S: string; Count: Integer): string;
var
  i: Integer;
begin
  if Count >= Length(S) then
  begin
    Result := S;
    Exit;
  end;
  i := 0;
  while i < Count do
  begin
    Inc(i);
    if S[i] in LeadBytes then
      Inc(i);
  end;
  if i > Count then
    Dec(i, 2);
  Result := Copy(S, 1, i);
end;

function RStr(const S: string; Count: Integer): string;
var
  i: Integer;
begin
  if Count >= Length(S) then
  begin
    Result := S;
    Exit;
  end;
  Count := Length(S) - Count;
  i := 0;
  while i < Count do
  begin
    Inc(i);
    if S[i] in LeadBytes then
      Inc(i);
  end;
  Result := Copy(S, i+1, Length(S)-i);
end;

function RevStr(const S: string): string;
var
  i, len: Integer;
begin
  len := Length(S);
  SetString(Result, nil, len);
  i := 0;
  while i < len do
  begin
    Inc(i);
    if S[i] in LeadBytes then
    begin
      Result[len-i] := S[i];
      Result[len-i+1] := S[i+1];
      Inc(i);
    end else
      Result[len-i+1] := S[i];
  end;
end;

function StrColl(const S1, S2: string): Integer;
begin
  Result := lstrcmp(PChar(S1), PChar(S2));
end;


{ Path Function }

const
  PathSep = '\';
  ExtSep = '.';
  DriveSep = ':';

function IsFile(const Path: string): Boolean;
var
  r: DWORD;
begin
  r := GetFileAttributes(PChar(Path));
  Result := (r <> $FFFFFFFF) and ((r and FILE_ATTRIBUTE_DIRECTORY) = 0);
end;

function IsDir(const Path: string): Boolean;
var
  r: DWORD;
begin
  r := GetFileAttributes(PChar(Path));
  Result := (r <> $FFFFFFFF) and ((r and FILE_ATTRIBUTE_DIRECTORY) <> 0);
end;

function AbsPath(const Path: string): string;
var
  buf: array [0..MAX_PATH] of Char;
  p: PChar;
  len: Integer;
begin
  len := GetFullPathName(PChar(Path), SizeOf(buf), buf, p);
  if (RStr(buf, 1) = PathSep) and (len > 3) then
    Dec(len);
  SetString(Result, buf, len);
end;

function ExpandVars(const Path: string): string;
var
  buf: array[0..MaxExpandVarsLen-1] of Char;
begin
  // Return value of ExpandEnviromentStrings is
  // Win2k:characters  Win9x:bytes. It is not useful. :-(
  ExpandEnvironmentStrings(PChar(Path), buf, SizeOf(buf));
  Result := buf;
end;

function GetEnv(const Name: string; const Default: string): string;
var
  buf: array[0..MaxGetEnvLen-1] of Char;
begin
  SetString(Result, Buf, GetEnvironmentVariable(PChar(Name), Buf,
    SizeOf(Buf)));
  if Result = '' then
    Result := Default;
end;

function SysError(Code: Integer): string;
var
  buf: PChar;
begin
  SetString(Result, buf, FormatMessage(
    FORMAT_MESSAGE_FROM_SYSTEM or FORMAT_MESSAGE_ALLOCATE_BUFFER or $FF,
    nil, Code, (SUBLANG_DEFAULT shl 10) or LANG_NEUTRAL, PChar(@buf), 0, nil));
  LocalFree(HLOCAL(buf));
  if Result = '' then
    Result := 'Unknown error: ' + IStr(Code);
end;

