unit SWinUtil;

interface

uses Windows;

function LocaleStr(lcid: LCID; LCType: LCTYPE): string;
function DateStr(const Date: TSystemTime): string;
function TimeStr(const Time: TSystemTime): string;

function OsVersionInfo: TOsVersionInfo;
function LongFileName(const FileName: string): string;
function GetFindData(const Path: string; var FindData: TWin32FindData): Boolean;

implementation

uses SUtil;

function LocaleStr(lcid: LCID; LCType: LCTYPE): string;
begin
  // -1: for null character
  SetString(Result, nil, GetLocaleInfo(lcid, LCType, nil, 0)-1);
  GetLocaleInfo(lcid, LCType, PChar(Result), Length(Result));
end;

function DateStr(const Date: TSystemTime): string;
begin
  SetString(Result, nil, GetDateFormat(LOCALE_USER_DEFAULT, DATE_LONGDATE,
    @Date, nil, nil, 0)-1); // -1: for null character
  GetDateFormat(LOCALE_USER_DEFAULT, DATE_LONGDATE, @Date, nil, PChar(Result),
                Length(Result));
end;

function TimeStr(const Time: TSystemTime): string;
begin
  SetString(Result, nil, GetTimeFormat(LOCALE_USER_DEFAULT, 0, @Time,
    nil, nil, 0)-1); // -1: for null character
  GetTimeFormat(LOCALE_USER_DEFAULT, 0, @Time, nil, PChar(Result),
    Length(Result));
end;

function OsVersionInfo: TOsVersionInfo;
begin
  Result.dwOSVersionInfoSize := SizeOf(Result);
  GetVersionEx(Result);
end;

function LongFileName(const FileName: string): string;
var
  h: THandle;
  fd: TWin32FindData;
  path: string;
begin
  Result := '';
  if FileName = '' then
    Exit;
  path := AbsPath(FileName);
  while True do
  begin
    h := FindFirstFile(PChar(path), fd);
    if h = INVALID_HANDLE_VALUE then
    begin
      Result := path + Result;
      Exit;
    end;
    FindClose(h);

    // FindFirstFile perhaps returned '.' before sharename (@Win95a)
    // (\\sharename\.\dir)
//    if string(fd.cFileName) = '.' then
    if fd.cFileName = PChar('.') then
    begin
      Result := path + Result;
      Exit;
    end;

    path := DirName(path);
    if (path = '\\') or ((Length(path) = 3) and (path[2] = ':')) then
    begin
      Result := path + fd.cFileName + Result;
      Exit;
    end;

//    Result := '\' + string(fd.cFileName) + Result;
    Result := string('\') + fd.cFileName + Result;
  end;
end;

function GetFindData(const Path: string;
  var FindData: TWin32FindData): Boolean;
var
  h: THandle;
begin
  FillChar(FindData, SizeOf(FindData), 0);
  h := FindFirstFile(PChar(Path), FindData);
  Result := h <> INVALID_HANDLE_VALUE;
  if Result then
    FindClose(h);
end;

end.

